@@ -25,6 +25,8 @@ Ted Pedersen <tpederse@d.umn.edu>
 
 Satanjeev Banerjee <banerjee@cs.cmu.edu>
 
+Bridget Thomson McInnes <bthomson@d.umn.edu>
+
 =head1 BUGS
 
 This measure currently only defined for bigram data stored in 2x2 
@@ -55,7 +57,30 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 =cut
 
 package	rightFisher;
+
+#  Make sure that measure2d.pm is available in the PATH. First
+#  we check in the directory you are running from, and then we
+#  look through the system path. If the module is not found anywhere
+#  then abort. 
+
+my $module = "measure2d.pm"; my $modulename = "measure2d.pm";
+
+if( !( -f $modulename ) ) {
+    my $found = 0;
+    #  Check each of the PATHS to see if the module is there
+    foreach (split(/:/, $ENV{PATH})) {
+ 	$module = $_ . "/" . $modulename;
+	if ( -f $module ) { $found = 1; last; }
+    }
+    # if still not found anywhere, quit!
+    if ( ! $found ) { print "Could not find $modulename.\n"; exit; }
+}
+
+# Include the module into the current package.    
+
+require $module;
 require	Exporter;
+
 @ISA = qw ( Exporter );
 @EXPORT = qw (initializeStatistic getStatisticName calculateStatistic errorCode errorString);
 
@@ -64,142 +89,35 @@ require	Exporter;
 # frequency combinations are enough to do the computation
 sub initializeStatistic
 {
-    ($ngram, $totalBigrams, $combIndex, @freqComb) = @_;
-    
-    $errorCodeNumber = 0;
-    $errorMessage = "";
-
-    # check if ngram > 2. Right-fisher statistic only defined for ngram = 2. 
-    if ($ngram > 2)
-    {
-	$errorCodeNumber = 1;
-	$errorMessage = "Right-fisher statistic is only available for bigrams!";
-	return;
-    }
-
-    # totalBigrams should not be less than equal to 0
-    if ($totalBigrams <= 0) 
-    { 
-	$errorCodeNumber = 10;
-	$errorMessage = "Total number of bigrams ($totalBigrams) must be greater than 0.";
-	return;
-    }
-
-    # figure out from the @freqComb array if the frequency values we
-    # are going to get are indeed the ones we need. the ones we need
-    # are (0,1), (0) and (1). while we figure this out, we shall also
-    # note which of the indices of the array passed to function
-    # calculateStatistic are the ones we want.
-
-    my $i;
-    for ($i = 0; $i < $combIndex; $i++)
-    {
-	$string = join (" ", @{$freqComb[$i]}[1..$freqComb[$i][0]]);
-
-	if ($string eq "0 1") { $jointFreqIndex = $i; }
-	elsif ($string eq "0") { $leftFreqIndex = $i; }
-	elsif ($string eq "1") { $rightFreqIndex = $i; }
-    }
-
-    if (!(defined $jointFreqIndex))
-    {
-	$errorCodeNumber = 100;
-	$errorMessage = "Frequency combination \"0 1\" (frequency of bigram) missing!\n";
-    }
-
-    if (!(defined $leftFreqIndex))
-    {
-	$errorCodeNumber = 101;
-	$errorMessage = "Frequency combination \"0\" (frequency of bigrams containing left token) missing!\n";
-    }
-
-    if (!(defined $rightFreqIndex))
-    {
-	$errorCodeNumber = 102;
-	$errorMessage = "Frequency combination \"1\" (frequency of bigrams containing right token) missing!\n";
-    }
+    measure2d::initializeStatistic(@_);
 }
 
 
 # function to calculate the left fisher value!
 sub calculateStatistic
 {
-    my @numbers = @_;
-    my $jointFrequency = $numbers[$jointFreqIndex];
-    my $leftFrequency  = $numbers[$leftFreqIndex];
-    my $rightFrequency = $numbers[$rightFreqIndex];
-
-    # joint frequency should be greater than equal to zero 
-    if ($jointFrequency < 0)
-    {
-	$errorCodeNumber = 200;
-	$errorMessage = "Frequency value ($jointFrequency) must not be negative.";
+    #  The parameters passed into calculateStatistic
+    #  need to be passed into getObservedValues for it 
+    #  to calculate the Observed values. If there is a 
+    #  problem with the observed values, this function 
+    #  will return, causing the measure to abort its 
+    #  computations and return with a zero code.
+    
+    if( !( ($n11, $n12, $n21, $n22) = measure2d::getObservedValues(@_) ) ) {
 	return(0);
     }
+    
+    #  Get the total number of bigrams; no need to check 
+    #  if this is correct because this value was already 
+    #  checked and computed in measure2d::initalizeStatistic.
+    #  getTotalBigrams simply reads an existing value. 
 
-    # joint frequency should be less than or equal to totalBigrams
-    if ($jointFrequency > $totalBigrams)
-    {
-	$errorCodeNumber = 201;
-	$errorMessage = "Frequency value ($jointFrequency) must not exceed total number of bigrams.";
-	return(0);
-    }
-
-    # joint frequency should be less than or equal to the marginal totals
-    if ($jointFrequency > $leftFrequency || $jointFrequency > $rightFrequency)
-    {
-	$errorCodeNumber = 202;
-	$errorMessage = "Frequency value of ngram ($jointFrequency) must not exceed the marginal totals.";
-	return(0);
-    }
-
-    # left frequency should be greater than or equal to zero 
-    if ($leftFrequency < 0)
-    {
-	$errorCodeNumber = 210;
-	$errorMessage = "Marginal total value ($leftFrequency) must not be negative.";
-	return(0);
-    }
-
-    # left frequency should be less than or equal to totalBigrams
-    if ($leftFrequency > $totalBigrams)
-    {
-	$errorCodeNumber = 211;
-	$errorMessage = "Marginal total value ($leftFrequency) must not exceed total number of bigrams.";
-	return(0);
-    }
-
-    # right frequency should be greater than or equal to zero 
-    if ($rightFrequency < 0)
-    {
-	$errorCodeNumber = 220;
-	$errorMessage = "Marginal total value ($rightFrequency) must not be negative.";
-	return(0);
-    }
-
-    # right frequency should be less than or equal to totalBigrams
-    if ($rightFrequency > $totalBigrams)
-    {
-	$errorCodeNumber = 221;
-	$errorMessage = "Marginal total value ($rightFrequency) must not exceed total number of bigrams.";
-	return(0);
-    }
-
-    # now the actual calculation!
-
-    my $n11 = $jointFrequency;
-    my $npp = $totalBigrams;
-    my $n1p = $leftFrequency;
-    my $np1 = $rightFrequency;
-    my $n2p = $npp - $n1p;
-    my $np2 = $npp - $np1;
-
-    # Added by Bridget
-    my $n12 = $n1p - $n11;
-    my $n21 = $np1 - $n11;
-    my $n22 = $np2 - $n12;
+    my $npp = measure2d::getTotalBigrams();
 
     
+    #  Get the marginal values
+    ($n1p, $np1, $n2p, $np2) = measure2d::getMarginalTotals();
+    
     # we shall have two	arrays one for the numerator and one for the
     # denominator. the arrays will contain the factorial upper limits. we
     # shall be arrange these two arrays	in descending order. while doing the
@@ -219,31 +137,12 @@ sub calculateStatistic
     # divide the result	for 0 with correct numbers to obtain result for	i,
     # i>0, i<=n11!! :o)
 
-    #  Bridget Thomson McInnes 15 October 2003
     #  Set the final limit to the least of the marginal totals
     #  n1p or np1 because that is the maximum number of times
     #  the bigram would be seen
 
     $final_Limit = ($n1p < $np1) ? $n1p : $np1;
 
-    ########### this part by Nitin O Varma
-
-    #$final_Limit = $n11;
-    #$n11  = 0;
-    #my $n12 = $n1p;
-    #my $n21 = $np1;
-    #my $n22 = $n2p - $n21;
-    
-    #while($n22 < 0)
-    #{
-    #	$n11++;
-    #    $n12 = $n1p - $n11;
-    #    $n21 = $np1 - $n11;
-    #    $n22 = $n2p - $n21;
-    #}
-
-    ########### end of part by Nitin O Varma
-
     my @denominator = sort { $b	<=> $a } ($npp,	$n22, $n12, $n21, $n11);
 
     # now that we have our two arrays all nicely sorted	and in place,
@@ -318,10 +217,7 @@ sub calculateStatistic
     $probability += $product;
 
     # now for the rest of n11's	!!
-    
-    #for ( $i = 1; $i <= $n11; $i++ )
-    
-    # Bridget Thomson McInnes 15 October 2003
+
     # Now we add the hypergeometric probability for the possible
     # n11 values greater than the original n11 up until the
     # final limit. 
@@ -351,18 +247,14 @@ sub calculateStatistic
 # error code. useful if the error can be recovered from!
 sub errorCode 
 { 
-    my $temp = $errorCodeNumber;
-    $errorCodeNumber = 0;
-    return($temp); 
+    return measure2d::errorCode();
 }
 
 # function to return the error message of the last operation and reset
 # the message string. useful if error can be recovered from!
 sub errorString
 {
-    my $temp = $errorMessage;
-    $errorMessage = "";
-    return($temp);
+    return measure2d::errorString();
 }
 
 # function to return the name of this statistic
